==> AxolotlClient-3.1.6%2B1.20.1-sources.jar: io/github/axolotlclient/modules/screenshotUtils/ImageScreen.java <==
/*
 * Copyright © 2024 moehreag <moehreag@gmail.com> & Contributors
 *
 * This file is part of AxolotlClient.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * For more information, see the LICENSE file.
 */

package io.github.axolotlclient.modules.screenshotUtils;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.time.Instant;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import net.minecraft.class_156;
import net.minecraft.class_2561;
import net.minecraft.class_310;
import net.minecraft.class_332;
import net.minecraft.class_339;
import net.minecraft.class_4185;
import net.minecraft.class_437;
import net.minecraft.class_5244;
import net.minecraft.class_6382;
import io.github.axolotlclient.AxolotlClient;
import io.github.axolotlclient.AxolotlClientCommon;
import io.github.axolotlclient.api.API;
import io.github.axolotlclient.api.util.UUIDHelper;
import io.github.axolotlclient.util.GsonHelper;
import io.github.axolotlclient.util.notifications.Notifications;

public class ImageScreen extends class_437 {

	private final class_437 parent;
	private final ImageInstance image;
	private final boolean freeOnClose;
	private final boolean isRemote;
	private final CompletableFuture<String> uploader;

	static class_437 create(class_437 parent, CompletableFuture<ImageInstance> future, boolean freeOnClose) {
		if (future.isDone()) {
			if (future.join() != null) {
				return new ImageScreen(parent, future.join(), freeOnClose);
			} else {
				return parent;
			}
		}
		return new LoadingImageScreen(parent, future.thenAccept(i -> {
			if (i != null) {
				class_310.method_1551().execute(() -> class_310.method_1551().method_1507(new ImageScreen(parent, i, freeOnClose)));
			} else {
				class_310.method_1551().execute(() -> class_310.method_1551().method_1507(parent));
			}
		}), freeOnClose);
	}

	private ImageScreen(class_437 parent, ImageInstance instance, boolean freeOnClose) {
		super(class_2561.method_43470(instance.filename()));
		this.parent = parent;
		this.image = instance;
		this.freeOnClose = freeOnClose;
		this.isRemote = image instanceof ImageInstance.Remote;
		if (isRemote) {
			this.uploader = UUIDHelper.tryGetUsernameAsync(((ImageInstance.Remote) image).uploader());
		} else {
			this.uploader = null;
		}
	}

	@Override
	public void method_25394(class_332 graphics, int mouseX, int mouseY, float delta) {
		method_25420(graphics);
		super.method_25394(graphics, mouseX, mouseY, delta);
		if (isRemote && uploader.isDone()) {
			ImageInstance.Remote r = (ImageInstance.Remote) image;
			graphics.method_27534(field_22793, method_25440(), field_22789 / 2, 38 / 2 - field_22793.field_2000 - 2, -1);
			graphics.method_27534(field_22793,
				class_2561.method_43469("gallery.image.upload_details", uploader.join(),
					r.sharedAt().atZone(ZoneId.systemDefault()).format(AxolotlClientCommon.getInstance().getConfig().getDateTimeFormatter())),
				field_22789 / 2, 38 / 2 + 2, -1);
		} else {
			graphics.method_27534(field_22793, method_25440(), field_22789 / 2, 33 / 2 - field_22793.field_2000 / 2, -1);
		}
	}

	@Override
	protected void method_25426() {

		int contentsHeight = field_22790 - ((isRemote ? 38 : 33) + 33);
		int buttonWidth = 75;
		double imgAspectRatio = image.image().method_4307() / (double) image.image().method_4323();
		int imageWidth = Math.min((int) (contentsHeight * imgAspectRatio), field_22789 - buttonWidth - 4 - 20);
		int imageHeight = (int) (imageWidth / imgAspectRatio);


		var element = method_37063(new ImageElement(imageWidth, imageHeight));
		if (field_22789 / 2 > (imageWidth / 2) + buttonWidth + 4) {
			element.method_48229(field_22789 / 2 - imageWidth / 2, 36);
		} else {
			element.method_48229(field_22789 / 2 - imageWidth / 2 - buttonWidth / 2 - 2, 36);
		}
		int actionX = element.method_46426() + imageWidth + 4;
		var actions = new ArrayList<class_4185>();
		if (image instanceof ImageInstance.Local local) {
			if (API.getInstance().isAuthenticated() && !(image instanceof ImageInstance.Remote)) {
				actions.add(class_4185.method_46430(class_2561.method_43471("gallery.image.upload"), b -> {
					b.field_22763 = false;
					ImageShare.getInstance().upload(local.location()).thenAccept(s -> {
						if (s.isEmpty()) {
							Notifications.getInstance().addStatus("gallery.image.upload.failure", "gallery.image.upload.failure.description");
						} else {
							client.execute(() -> client.setScreen(new ImageScreen(parent, local.toShared(s, API.getInstance().getSelf().getUuid(), Instant.now()), freeOnClose)));
							client.keyboard.setClipboard(s);
							Notifications.getInstance().addStatus("gallery.image.upload.success", "gallery.image.upload.success.description", s);
						}
					});
				}).method_46432(buttonWidth).method_46431());
			}
			actions.add(class_4185.method_46430(class_2561.method_43471("gallery.image.copy"), b -> ScreenshotCopying.copy(local.location())).method_46432(buttonWidth).method_46431());
			actions.add(class_4185.method_46430(class_2561.method_43471("gallery.image.open.external"), b -> class_156.method_668().method_673(local.location().toUri())).method_46432(buttonWidth).method_46431());
		}
		if (image instanceof ImageInstance.Remote remote) {
			if (!(image instanceof ImageInstance.Local)) {
				actions.add(class_4185.method_46430(class_2561.method_43471("gallery.image.save"), b -> {
					b.field_22763 = false;
					try {
						Path out = saveSharedImage(remote);
						field_22787.method_1507(new ImageScreen(parent, remote.toShared(out), freeOnClose));
					} catch (IOException e) {
						Notifications.getInstance().addStatus("gallery.image.save.failure", "gallery.image.save.failure.description", e.getMessage());
						AxolotlClient.LOGGER.warn("Failed to save shared image!", e);
					}
				}).method_46432(buttonWidth).method_46431());
				actions.add(class_4185.method_46430(class_2561.method_43471("gallery.image.copy"), b -> {
					try {
						ScreenshotCopying.copy(image.image().method_24036());
					} catch (IOException e) {
						Notifications.getInstance().addStatus("gallery.image.copy.failure", "gallery.image.copy.failure.description", e.getMessage());
						AxolotlClient.LOGGER.warn("Failed to copy shared image!", e);
					}
				}).method_46432(buttonWidth).method_46431());
			}
			actions.add(class_4185.method_46430(class_2561.method_43471("gallery.image.open.external.browser"), b -> class_156.method_668().method_670(remote.url())).method_46432(buttonWidth).method_46431());
			actions.add(class_4185.method_46430(class_2561.method_43471("gallery.image.copy_url"), b -> field_22787.field_1774.method_1455(remote.url())).method_46432(buttonWidth).method_46431());
		}
		int actionY = element.method_46427();
		for (class_4185 w : actions) {
			w.method_48229(actionX, actionY);
			method_37063(w);
			actionY += 4 + w.method_25364();
		}

		method_37063(class_4185.method_46430(class_5244.field_24339, b -> method_25419()).method_46434(field_22789 / 2 - 75, field_22790 - 33 + 33 / 2 - 10, 150, 20).method_46431());
	}

	@Override
	public void method_25419() {
		if (freeOnClose) {
			field_22787.method_1531().method_4615(image.id());
		}
		field_22787.method_1507(parent);
	}

	private Path saveSharedImage(ImageInstance.Remote remote) throws IOException {
		Path out = GalleryScreen.SCREENSHOTS_DIR.resolve("shared")
			.resolve(remote.uploader())
			.resolve(remote.filename());
		Path infoJson = out.resolveSibling(remote.filename() + ".json");

		Files.createDirectories(out.getParent());
		remote.image().method_4314(out);
		Object json = Map.of("uploader", remote.uploader(), "shared_at", remote.sharedAt());
		Files.writeString(infoJson, GsonHelper.GSON.toJson(json));
		return out;
	}

	private class ImageElement extends class_339 {

		public ImageElement(int width, int height) {
			super(0, 0, width, height, class_2561.method_43473());
			field_22763 = false;
		}

		@Override
		protected void method_48579(class_332 guiGraphics, int mouseX, int mouseY, float partialTick) {
			guiGraphics.method_25290(image.id(), method_46426(), method_46427(), 0, 0, method_25368(), method_25364(), method_25368(), method_25364());
		}

		@Override
		protected void method_47399(class_6382 builder) {

		}
	}
}

==> AxolotlClient-3.1.8%2B1.21.1-sources.jar: io/github/axolotlclient/modules/screenshotUtils/ImageScreen.java <==
/*
 * Copyright © 2024 moehreag <moehreag@gmail.com> & Contributors
 *
 * This file is part of AxolotlClient.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * For more information, see the LICENSE file.
 */

package io.github.axolotlclient.modules.screenshotUtils;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.time.Instant;
import java.time.ZoneId;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import net.minecraft.class_156;
import net.minecraft.class_2561;
import net.minecraft.class_310;
import net.minecraft.class_332;
import net.minecraft.class_339;
import net.minecraft.class_4185;
import net.minecraft.class_437;
import net.minecraft.class_5244;
import net.minecraft.class_6382;
import net.minecraft.class_7842;
import net.minecraft.class_7852;
import net.minecraft.class_8132;
import net.minecraft.class_8667;
import io.github.axolotlclient.AxolotlClient;
import io.github.axolotlclient.AxolotlClientCommon;
import io.github.axolotlclient.api.API;
import io.github.axolotlclient.api.util.UUIDHelper;
import io.github.axolotlclient.util.GsonHelper;
import io.github.axolotlclient.util.notifications.Notifications;

public class ImageScreen extends class_437 {

	private final class_437 parent;
	private final ImageInstance image;
	private final boolean freeOnClose;

	static class_437 create(class_437 parent, CompletableFuture<ImageInstance> future, boolean freeOnClose) {
		if (future.isDone()) {
			if (future.join() != null) {
				return new ImageScreen(parent, future.join(), freeOnClose);
			} else {
				return parent;
			}
		}
		return new LoadingImageScreen(parent, future.thenAccept(i -> {
			if (i != null) {
				class_310.method_1551().execute(() -> class_310.method_1551().method_1507(new ImageScreen(parent, i, freeOnClose)));
			} else {
				class_310.method_1551().execute(() -> class_310.method_1551().method_1507(parent));
			}
		}), freeOnClose);
	}

	private ImageScreen(class_437 parent, ImageInstance instance, boolean freeOnClose) {
		super(class_2561.method_43470(instance.filename()));
		this.parent = parent;
		this.image = instance;
		this.freeOnClose = freeOnClose;
	}

	@Override
	protected void method_25426() {
		class_8132 layout = new class_8132(this);
		class_8667 header = layout.method_48992(class_8667.method_52741()).method_52735(4);
		header.method_52740().method_46467();
		header.method_52736(new class_7842(method_25440(), field_22793));

		if (image instanceof ImageInstance.Remote remote) {
			layout.method_48995(38);
			var uploader = header.method_52736(new class_7842(class_2561.method_43469("gallery.image.upload_details", "", remote.sharedAt().atZone(ZoneId.systemDefault()).format(AxolotlClientCommon.getInstance().getConfig().getDateTimeFormatter())), field_22793));
			UUIDHelper.tryGetUsernameAsync(remote.uploader()).thenAccept(name -> {
				uploader.method_25355(class_2561.method_43469("gallery.image.upload_details", name, remote.sharedAt().atZone(ZoneId.systemDefault()).format(AxolotlClientCommon.getInstance().getConfig().getDateTimeFormatter())));
				uploader.method_25358(field_22793.method_27525(uploader.method_25369()));
			});
		}

		int buttonWidth = 75;
		double imgAspectRatio = image.image().method_4307() / (double) image.image().method_4323();
		int imageWidth = Math.min((int) (layout.method_57727() * imgAspectRatio), layout.method_25368() - buttonWidth - 4 - 20);
		int imageHeight = (int) (imageWidth / imgAspectRatio);

		var contents = layout.method_48999(class_8667.method_52742().method_52735(4));
		if (field_22789 / 2 > (imageWidth / 2) + buttonWidth + 4) {
			contents.method_52736(new class_7852(buttonWidth + 4, imageHeight));
		}
		var footer = layout.method_48996(class_8667.method_52742().method_52735(4));
		contents.method_52736(new ImageElement(imageWidth, imageHeight));
		var actions = contents.method_52736(class_8667.method_52741()).method_52735(4);
		if (image instanceof ImageInstance.Local local) {
			if (API.getInstance().isAuthenticated() && !(image instanceof ImageInstance.Remote)) {
				actions.method_52736(class_4185.method_46430(class_2561.method_43471("gallery.image.upload"), b -> {
					b.field_22763 = false;
					ImageShare.getInstance().upload(local.location()).thenAccept(s -> {
						if (s.isEmpty()) {
							Notifications.getInstance().addStatus("gallery.image.upload.failure", "gallery.image.upload.failure.description");
						} else {
							client.execute(() -> client.setScreen(new ImageScreen(parent, local.toShared(s, API.getInstance().getSelf().getUuid(), Instant.now()), freeOnClose)));
							client.keyboard.setClipboard(s);
							Notifications.getInstance().addStatus("gallery.image.upload.success", "gallery.image.upload.success.description", s);
						}
					});
				}).method_46432(buttonWidth).method_46431());
			}
			actions.method_52736(class_4185.method_46430(class_2561.method_43471("gallery.image.copy"), b -> ScreenshotCopying.copy(local.location())).method_46432(buttonWidth).method_46431());
			actions.method_52736(class_4185.method_46430(class_2561.method_43471("gallery.image.open.external"), b -> class_156.method_668().method_673(local.location().toUri())).method_46432(buttonWidth).method_46431());
		}
		if (image instanceof ImageInstance.Remote remote) {
			if (!(image instanceof ImageInstance.Local)) {
				actions.method_52736(class_4185.method_46430(class_2561.method_43471("gallery.image.save"), b -> {
					b.field_22763 = false;
					try {
						Path out = saveSharedImage(remote);
						field_22787.method_1507(new ImageScreen(parent, remote.toShared(out), freeOnClose));
					} catch (IOException e) {
						Notifications.getInstance().addStatus("gallery.image.save.failure", "gallery.image.save.failure.description", e.getMessage());
						AxolotlClient.LOGGER.warn("Failed to save shared image!", e);
					}
				}).method_46432(buttonWidth).method_46431());
				actions.method_52736(class_4185.method_46430(class_2561.method_43471("gallery.image.copy"), b -> {
					try {
						ScreenshotCopying.copy(image.image().method_24036());
					} catch (IOException e) {
						Notifications.getInstance().addStatus("gallery.image.copy.failure", "gallery.image.copy.failure.description", e.getMessage());
						AxolotlClient.LOGGER.warn("Failed to copy shared image!", e);
					}
				}).method_46432(buttonWidth).method_46431());
			}
			actions.method_52736(class_4185.method_46430(class_2561.method_43471("gallery.image.open.external.browser"), b -> class_156.method_668().method_670(remote.url())).method_46432(buttonWidth).method_46431());
			actions.method_52736(class_4185.method_46430(class_2561.method_43471("gallery.image.copy_url"), b -> field_22787.field_1774.method_1455(remote.url())).method_46432(buttonWidth).method_46431());
		}

		footer.method_52736(class_4185.method_46430(class_5244.field_24339, b -> method_25419()).method_46431());

		layout.method_48222();
		layout.method_48206(this::method_37063);
	}

	@Override
	public void method_25419() {
		if (freeOnClose) {
			field_22787.method_1531().method_4615(image.id());
		}
		field_22787.method_1507(parent);
	}

	private Path saveSharedImage(ImageInstance.Remote remote) throws IOException {
		Path out = GalleryScreen.SCREENSHOTS_DIR.resolve("shared")
			.resolve(remote.uploader())
			.resolve(remote.filename());
		Path infoJson = out.resolveSibling(remote.filename() + ".json");

		Files.createDirectories(out.getParent());
		remote.image().method_4314(out);
		Object json = Map.of("uploader", remote.uploader(), "shared_at", remote.sharedAt());
		Files.writeString(infoJson, GsonHelper.GSON.toJson(json));
		return out;
	}

	private class ImageElement extends class_339 {

		public ImageElement(int width, int height) {
			super(0, 0, width, height, class_2561.method_43473());
			field_22763 = false;
		}

		@Override
		protected void method_48579(class_332 guiGraphics, int mouseX, int mouseY, float partialTick) {
			guiGraphics.method_25290(image.id(), method_46426(), method_46427(), 0, 0, method_25368(), method_25364(), method_25368(), method_25364());
		}

		@Override
		protected void method_47399(class_6382 builder) {

		}
	}
}
